from masonite.controllers import Controller
from masonite.request import Request
from masonite.response import Response
from masonite.views import View
from ..filemanager import FileManager


class FileManagerController(Controller):
    def __init__(self, response: Response, request: Request) -> None:
        self.response = response
        self.request = request
        self.manager = FileManager()

    def index(self, view: View):
        return view.render("filemanager:index", {})

    def all_files(self):
        return self.manager.provider().all_files()

    def upload(self):
        file = self.request.input("file")
        self.manager.provider().upload(file)

        return self.response.json(
            {
                "message": "File uploaded!",
            }
        )

    def rename(self):
        name = self.request.input("name")
        path = self.request.input("path")
        self.manager.provider().rename(path, name)

        return self.response.json(
            {
                "message": "File/Folder renamed!",
            }
        )

    def create_folder(self):
        name = self.request.input("name")

        if not self.manager.provider().exists(name):
            self.manager.provider().create_folder(name)

            return self.response.json(
                {
                    "message": "Folder Created...",
                }
            )

        return self.response.json(
            {
                "message": "Folder already exists!",
            }
        )

    def delete_folder(self):
        path = self.request.input("path")

        try:
            self.manager.provider().delete_folder(path)

            return self.response.json(
                {
                    "message": "Folder deleted...",
                }
            )
        except Exception as e:
            print(e)

        return self.response.json(
            {
                "message": "Folder doesn't exists!",
            }
        )

    def delete_file(self):
        path = self.request.input("path")

        try:
            self.manager.provider().delete_file(path)
            return self.response.json(
                {
                    "message": "File deleted...",
                }
            )
        except Exception as e:
            print(e)

        return self.response.json(
            {
                "message": "File doesn't exists!",
            }
        )

    def picker(self, view: View):
        return view.render("filemanager:picker", {})



from masonite.environment import LoadEnvironment, env
from masoniteorm.connections import ConnectionResolver

#  Loads in the environment variables when this page is imported.
LoadEnvironment()

"""
The connections here don't determine the database but determine the "connection".
They can be named whatever you want.
"""
DATABASES = {
    "default": env("DB_CONNECTION", "sqlite"),
    "sqlite": {
        "driver": "sqlite",
        "database": env("SQLITE_DB_DATABASE", "masonite.sqlite3"),
        "prefix": "",
        "log_queries": env("DB_LOG"),
    },
    "mysql": {
        "driver": "mysql",
        "host": env("DB_HOST"),
        "user": env("DB_USERNAME"),
        "password": env("DB_PASSWORD"),
        "database": env("DB_DATABASE"),
        "port": env("DB_PORT"),
        "prefix": "",
        "grammar": "mysql",
        "options": {
            "charset": "utf8mb4",
        },
        "log_queries": env("DB_LOG"),
    },
    "postgres": {
        "driver": "postgres",
        "host": env("DB_HOST"),
        "user": env("DB_USERNAME"),
        "password": env("DB_PASSWORD"),
        "database": env("DB_DATABASE"),
        "port": env("DB_PORT"),
        "prefix": "",
        "grammar": "postgres",
        "log_queries": env("DB_LOG"),
    },
    "mssql": {
        "driver": "mssql",
        "host": env("MSSQL_DATABASE_HOST"),
        "user": env("MSSQL_DATABASE_USER"),
        "password": env("MSSQL_DATABASE_PASSWORD"),
        "database": env("MSSQL_DATABASE_DATABASE"),
        "port": env("MSSQL_DATABASE_PORT"),
        "prefix": "",
        "log_queries": env("DB_LOG"),
    },
}

DB = ConnectionResolver().set_connection_details(DATABASES)



from masonite.routes import Route

from ..controllers.filemanager_controller import FileManagerController


ROUTES = Route.group(
    [
        Route.get("/filemanager", FileManagerController.index),
        Route.post("/filemanager", FileManagerController.upload),
        Route.get("/filemanager/all-files", FileManagerController.all_files),
        Route.post("/filemanager/rename", FileManagerController.rename),
        Route.post("/filemanager/create-folder", FileManagerController.create_folder),
        Route.post("/filemanager/delete-folder", FileManagerController.delete_folder),
        Route.post("/filemanager/delete-file", FileManagerController.delete_file),
        Route.get("/filemanager/picker", FileManagerController.picker),
    ]
)



from masonite.providers import (
    RouteProvider,
    FrameworkProvider,
    ViewProvider,
    WhitenoiseProvider,
    ExceptionProvider,
    MailProvider,
    SessionProvider,
    QueueProvider,
    CacheProvider,
    EventProvider,
    StorageProvider,
    HelpersProvider,
    BroadcastProvider,
    AuthenticationProvider,
    AuthorizationProvider,
    HashServiceProvider,
    ORMProvider,
)


from masonite.scheduling.providers import ScheduleProvider
from masonite.notification.providers import NotificationProvider
from masonite.validation.providers import ValidationProvider

from src.filemanager.providers import FileManagerProvider

PROVIDERS = [
    FrameworkProvider,
    HelpersProvider,
    RouteProvider,
    ViewProvider,
    WhitenoiseProvider,
    ExceptionProvider,
    MailProvider,
    NotificationProvider,
    SessionProvider,
    CacheProvider,
    QueueProvider,
    ScheduleProvider,
    EventProvider,
    StorageProvider,
    BroadcastProvider,
    HashServiceProvider,
    AuthenticationProvider,
    ValidationProvider,
    AuthorizationProvider,
    ORMProvider,
    FileManagerProvider,
]



from masonite.foundation import response_handler
from masonite.storage import StorageCapsule
from masonite.auth import Sign
from masonite.environment import LoadEnvironment
from masonite.utils.structures import load
from masonite.utils.location import base_path
from masonite.middleware import (
    SessionMiddleware,
    EncryptCookies,
    LoadUserMiddleware,
    MaintenanceModeMiddleware,
)
from masonite.routes import Route
from masonite.configuration.Configuration import Configuration
from masonite.configuration import config

from .app.middlewares.VerifyCsrfToken import VerifyCsrfToken


class Kernel:

    http_middleware = [MaintenanceModeMiddleware, EncryptCookies]

    route_middleware = {
        "web": [SessionMiddleware, LoadUserMiddleware, VerifyCsrfToken],
    }

    def __init__(self, app):
        self.application = app

    def register(self):
        # Register routes
        self.load_environment()
        self.register_configurations()
        self.register_middleware()
        self.register_routes()
        self.register_database()
        self.register_templates()
        self.register_storage()

    def load_environment(self):
        LoadEnvironment()

    def register_configurations(self):
        # load configuration
        self.application.bind("config.location", "tests/integrations/config")
        configuration = Configuration(self.application)
        configuration.load()
        self.application.bind("config", configuration)
        key = config("application.key")
        self.application.bind("key", key)
        self.application.bind("sign", Sign(key))
        # set locations
        self.application.bind("resources.location", "tests/integrations/resources/")
        self.application.bind("controllers.location", "tests/integrations/app/controllers")
        self.application.bind("jobs.location", "tests/integrations/jobs")
        self.application.bind("providers.location", "tests/integrations/providers")
        self.application.bind("mailables.location", "tests/integrations/mailables")
        self.application.bind("listeners.location", "tests/integrations/listeners")
        self.application.bind("validation.location", "tests/integrations/validation")
        self.application.bind("notifications.location", "tests/integrations/notifications")
        self.application.bind("events.location", "tests/integrations/events")
        self.application.bind("tasks.location", "tests/integrations/tasks")
        self.application.bind("models.location", "tests/integrations/app/models")
        self.application.bind("observers.location", "tests/integrations/models/observers")
        self.application.bind("policies.location", "tests/integrations/policies")
        self.application.bind("commands.location", "tests/integrations/commands")
        self.application.bind("middlewares.location", "tests/integrations/app/middlewares")

        self.application.bind("server.runner", "masonite.commands.ServeCommand.main")

    def register_middleware(self):
        self.application.make("middleware").add(self.route_middleware).add(self.http_middleware)

    def register_routes(self):
        Route.set_controller_locations(self.application.make("controllers.location"))
        self.application.bind("routes.location", "tests/integrations/routes/web")
        self.application.make("router").add(
            Route.group(
                load(self.application.make("routes.location"), "ROUTES"), middleware=["web"]
            )
        )

    def register_database(self):
        from masoniteorm.query import QueryBuilder

        self.application.bind(
            "builder",
            QueryBuilder(connection_details=config("database.databases")),
        )

        self.application.bind("migrations.location", "tests/integrations/databases/migrations")
        self.application.bind("seeds.location", "tests/integrations/databases/seeds")

        self.application.bind("resolver", config("database.db"))

    def register_templates(self):
        self.application.bind("views.location", "tests/integrations/templates/")

    def register_storage(self):
        storage = StorageCapsule()
        storage.add_storage_assets(config("filesystem.staticfiles"))
        self.application.bind("storage_capsule", storage)

        self.application.set_response_handler(response_handler)
        self.application.use_storage_path(base_path("storage"))



from masonite.configuration import config
from .drivers.local_driver import LocalDriver
from .drivers.s3_driver import S3Driver


class FileManager:
    def __init__(self) -> None:
        self.conf = config("filemanager")

    def provider(self, driver=None):
        if driver is None:
            driver = self.conf.get("driver", "local")
        if driver == "local":
            return LocalDriver()
        if driver == "s3":
            return S3Driver()

        raise ModuleNotFoundError("Driver not found")



from masoniteorm.migrations import Migration


class CreatePasswordResetTable(Migration):
    def up(self):
        """Run the migrations."""
        with self.schema.create("password_resets") as table:
            table.string("email").unique()
            table.string("token")
            table.datetime("expires_at").nullable()
            table.datetime("created_at")

    def down(self):
        """Revert the migrations."""
        self.schema.drop("password_resets")



from masonite.foundation import Application, Kernel
from tests.integrations.config.providers import PROVIDERS
from tests.integrations.Kernel import Kernel as ApplicationKernel


"""Start The Application Instance."""
application = Application("tests/integrations")

"""Now Bind important providers needed to make the framework work."""
application.register_providers(Kernel, ApplicationKernel)

"""Now Bind important application specific providers needed to make the application work."""
application.add_providers(*PROVIDERS)
